import { App, Modal } from "obsidian";
import { mount, unmount, type Component, type ComponentProps } from "svelte";

export class SvelteModal<
	TComponent extends Component<Record<string, any>, any, any>
> extends Modal {
	protected component: TComponent | undefined;

	constructor(
		app: App,
		private readonly componentCtor: TComponent,
		private readonly props: ComponentProps<TComponent>
	) {
		super(app);
	}

	onOpen(): void {
		this.component = mount(this.componentCtor, {
			target: this.contentEl,
			props: this.props,
		});
	}

	onClose(): void {
		if (this.component) {
			unmount(this.component);
			this.component = undefined;
		}
	}
}
