import { normalizePath, type Vault } from "obsidian";

/**
 * A simple analog of Node.js's `path.join(...)`.
 * See: https://gist.github.com/creationix/7435851#gistcomment-3698888
 */
export default function joinPath(...segments: string[]): string {
	const parts = segments.reduce((parts, segment) => {
		// Remove leading slashes from non-first part.
		if (parts.length > 0) {
			segment = segment.replace(/^\//, "");
		}
		// Remove trailing slashes.
		segment = segment.replace(/\/$/, "");
		return parts.concat(segment.split("/"));
	}, [] as string[]);
	const resultParts: string[] = [];
	for (const part of parts) {
		if (part === ".") {
			continue;
		}
		if (part === "..") {
			resultParts.pop();
			continue;
		}
		resultParts.push(part);
	}
	return resultParts.join("/");
}

export function dirname(path: string): string {
	return joinPath(path, "..");
}

export async function mkdirRecursive(
	vault: Vault,
	path: string
): Promise<void> {
	const stack: string[] = [];
	let currentPath = normalizePath(path);

	if (await vault.adapter.exists(currentPath)) {
		return;
	}

	while (!(await vault.adapter.exists(currentPath))) {
		stack.push(currentPath);
		currentPath = dirname(currentPath);
	}

	while (stack.length > 0) {
		await vault.adapter.mkdir(stack.pop()!);
	}
}
